#!/usr/bin/python

__doc__ = """Boost Software License - Version 1.0 - August 17th, 2003

Permission is hereby granted, free of charge, to any person or organization
obtaining a copy of the software and accompanying documentation covered by
this license (the "Software") to use, reproduce, display, distribute,
execute, and transmit the Software, and to prepare derivative works of the
Software, and to permit third-parties to whom the Software is furnished to
do so, all subject to the following:

The copyright notices in the Software and this entire statement, including
the above license grant, this restriction and the following disclaimer,
must be included in all copies of the Software, in whole or in part, and
all derivative works of the Software, unless such copies or derivative
works are solely in the form of machine-executable object code generated by
a source language processor.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE, TITLE AND NON-INFRINGEMENT. IN NO EVENT
SHALL THE COPYRIGHT HOLDERS OR ANYONE DISTRIBUTING THE SOFTWARE BE LIABLE
FOR ANY DAMAGES OR OTHER LIABILITY, WHETHER IN CONTRACT, TORT OR OTHERWISE,
ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
DEALINGS IN THE SOFTWARE."""

import os
#import botfather
#import finder
#import results
#import playerstat
from bots import *
import copy

verbose = False
#verbose = True

#info = False
info = True

#superverbose = verbose and False
superverbose = verbose and True

class xenophon(botfather.botfparam):
    def __init__(self,path):
        botfather.botfparam.__init__(self,path)        

    def calculateweight(self):
        weight = {}
	weight['Bomb'] = 1.0
	weight['R'] = 2.0 / 3.0
	weight['Y'] = 6.0 / 15.0
	weight['G'] = 1.0 / 5.0
	weight['L'] = self.param['flilac']
	weight['B'] = self.param['fblue']	
	return weight

    def calcdiff(self,playermove,opponentmove):

        opresult = results.resultsopponent(self.ostats,self.pstats,self.param['foshield'],self.param['folife'])
	tempfield = copy.deepcopy(playermove['field'])
	tempfield.makemoveandreduce(opponentmove['move'],opresult)
	opresult.eval()
	deadlyresult = self.isplayerkill(opresult,playermove['result'])

	#print "Opponent field after reduce %s:\n %s" % (opresult, tempfield)

	if superverbose: print "Damage by opponent: %d %s %s\nDamage by player  : %d %s" % (opresult.calcdamage(self.ostats),opponentmove['move'],opresult, playermove['result'].calcdamage(self.pstats),playermove['move'])
	return (playermove['result'].value - opresult.value,deadlyresult)

    def evalmove(self,field,result,move):
        tempfield = copy.deepcopy(field)
        tempresults = copy.deepcopy(result)
	tempfield.makemoveandreduce(move['move'],tempresults)
	tempresults.eval()

	return tempfield,tempresults

    def makepanicmove(self):
        self.move = (1,1,0,1)
        if info or verbose: print "Move (neg %.2f Deadly: %s ): %s with result: %s" % (-1000.0,True,str(self.move), "Don't know")


    def calcmove(self):
        findmoves = finder.finder(self.field)
	if verbose: print findmoves

        weights = self.calculateweight()
	listresults = []
	templateresult = results.resultsblue(self.pstats,self.ostats,self.param['fpshield'],self.param['fplife'],weights['Bomb'],weights['R'],weights['Y'],weights['G'],weights['B'],weights['L'])

	if verbose: print self.field
	if verbose: print "Moves: " , len(findmoves.moves)
	for move in findmoves.moves:
	    tempfield,tempresults = self.evalmove(self.field,templateresult,move)

	    #Check if this is a special move.
	    #if it we can stop searching for other moves
	    if self.isdeadlyimpact(tempresults):
	        self.move = move['move']
	        if info or verbose: print "Move (deadly): %s with result: %s" % (str(self.move), tempresults)
		return
	    elif self.giveextramove(tempresults):
	        #We have an extra move so we can reduce the gamefield again
                findlilacmoves = finder.finder(tempfield)
		listlilacresults = []

	        for lilacmove in findlilacmoves.moves:
		    templilacfield,templilacresults = self.evalmove(tempfield,tempresults,lilacmove)
                    
	            listlilacresults.append({'result' : templilacresults, 'move' : lilacmove, 'field' : templilacfield})
	        listlilacresults.sort(lambda first,second: int(second['result'].value * 1000 - first['result'].value * 1000 ))
		tempresults = listlilacresults[0]['result']
		tempresults.islilac = True
		tempfield   = listlilacresults[0]['field']

	    listresults.append({'result' : tempresults, 'move' : move['move'], 'field' : tempfield,'deadly' : False})
	    
	
	listresults.sort(lambda first,second: int(second['result'].value * 1000 - first['result'].value * 1000 ))
	if superverbose:
	    for result in listresults:
	        print "%5.2f/%3d : %s --> %s" %(result['result'].value,result['result'].calcdamage(self.pstats),str(result['result']),str(result['move']))
	
	maxdiffdamage = -1000
	positvemoves = []
	deadlymoves = []

	for index,moveresult in enumerate(listresults):
		opmoves = finder.finder(moveresult['field'])
		negativvalues = []
		moveresult['opponent'] = {'moves' : opmoves, 'diffs' : [], 'alreadychecked' : []}
		for move in opmoves.moves:
		    diff,deadlyresult = self.calcdiff(moveresult,move)
                    moveresult['opponent']['diffs'].append(diff)
                    moveresult['opponent']['alreadychecked'].append(move)
		    moveresult['deadly'] |= deadlyresult

		    if diff < 0.0:
			break

		else:
		    try:
			if moveresult['deadly'] == False:
		            if verbose: print "Append move %s with diff %.2f" % (moveresult['move'],diff)
		            positvemoves.append({'result' : moveresult, 'diff': diff})
			else:
			    deadlymoves.append(moveresult)
		    except UnboundLocalError:
		        #if diff is not defined no opponent move is possible then we have a ultra positive move
		        positvemoves.append({'result' : moveresult, 'diff': moveresult['result'].value})
		        
				
        else:
	    if len(positvemoves) > 0:
                positvemoves.sort(lambda first,second: int(second['diff'] - first['diff']))
	        self.move = positvemoves[0]['result']['move']
	        if info or verbose: print "Move (pos %.2f Deadly: %s): %s with result: %s" % (positvemoves[0]['diff'],positvemoves[0]['result']['deadly'],str(self.move), positvemoves[0]['result']['result'])
	    else:
	        for move in deadlymoves: listresults.remove(move)
	        globalmindiff = -10000
	        for index,moveresult in enumerate(listresults):
		    for index,move in enumerate(moveresult['opponent']['moves'].moves):
		        if move in moveresult['opponent']['alreadychecked']:
			    #print "Leave out! %s" % str(move['move'])
			    if moveresult['opponent']['diffs'][index] < globalmindiff:
			        break
			    else:
			        continue
			else:
		            diff,deadlyresult = self.calcdiff(moveresult,move)

                            moveresult['opponent']['diffs'].append(diff)
                            moveresult['opponent']['alreadychecked'].append(move)
			    moveresult['deadly'] |= deadlyresult

			    if diff < globalmindiff:
			        break
		    else:
		        moveresult['opponent']['diffs'].sort(lambda first,second: int(first * 1000 - second * 1000))
			if moveresult['opponent']['diffs'][0] > globalmindiff:
			    globalmindiff = moveresult['opponent']['diffs'][0]

	        for result in listresults:
		    result['opponent']['diffs'].sort(lambda first,second: int(first * 1000 - second * 1000))
		    #print result['opponent']['diffs']
		
	        listresults.sort(lambda first,second: int(second['opponent']['diffs'][0] * 1000 - first['opponent']['diffs'][0] * 1000 ))
		if len(listresults) > 0:
		    for move in listresults:
		        if move['deadly'] == False:
	                    self.move = listresults[0]['move']
	                    if info or verbose: print "Move (neg %.2f Deadly: %s ): %s with result: %s" % (move['opponent']['diffs'][0],move['deadly'],str(self.move), move['result'])
			    break
		    else:
		        self.makepanicmove()
		else:
		    self.makepanicmove()
	#print listresults
	#self.move = listresults[0]['move']

if __name__ == "__main__":
    test = xenophon(os.getcwd())
    test.makemove()
    test.writemove()

    print test
